@@ -43,8 +43,11 @@
 #include "quantacommon.h"
 #include "document.h"
 #include "qextfileinfo.h"
+
+
 #include "kafkacommon.h"
 #include "undoredo.h"
+
 #include "dtds.h"
 #include "structtreetag.h"
 
@@ -64,6 +67,9 @@
 extern GroupElementMapList globalGroupMap;
 static const QChar space(' ');
 
+extern int NN;
+extern QValueList<Node*> nodes;
+
 Parser::Parser()
 {
   m_node = 0L;
@@ -457,7 +463,7 @@ Node *Parser::parseArea(int startLine, int startCol, int endLine, int endCol, No
         ec = 0;
     AreaStruct area(el, ec, endLine, endCol);
 #ifdef DEBUG_PARSER
-    kdDebug(24000) << "Calling cleanGroups from Parser::parseArea" << endl;
+//     kdDebug(24000) << "Calling cleanGroups from Parser::parseArea" << endl;
 #endif
     cleanGroups();
     m_saParser->setParsingEnabled(true);
@@ -515,10 +521,18 @@ Node *Parser::parse(Document *w, bool force)
  // clearGroups();
   if (baseNode)
   {
-    delete baseNode;
-    baseNode = 0L;
-    m_node = 0L;
+     kdDebug(24000) << "Node objects before delete = " << NN << " ; list count = " << nodes.count() << endl;
+     //kdDebug(24000) << "baseNode before delete = " << baseNode << endl;
+     //ParserCommon::coutTree(m_node, 2);
+     delete baseNode;
+     baseNode = 0L;
+     kdDebug(24000) << "Node objects after delete = " << NN << " ; list count = " << nodes.count() << endl;
+     QValueList<Node*> nList = nodes;
+/*     for (QValueList<Node*>::ConstIterator it = nList.constBegin(); it != nList.constEnd(); ++it)
+       delete (*it);
+     kdDebug(24000) << "Node objects after cleanup = " << NN << " ; list count = " << nodes.count() << endl;       */
   }
+  m_node = 0L;
 
   Node *lastNode;
   write = w;
@@ -535,14 +549,15 @@ Node *Parser::parse(Document *w, bool force)
     m_node = ParserCommon::createTextNode(w, 0L, maxLines, w->editIf->lineLength(maxLines), 0L);
   }
   m_parsingNeeded = false;
-/*
- treeSize = 0;
- kdDebug(24000) << "Basenode : " << m_node << endl;
- coutTree(m_node, 2);
- kdDebug(24000) << "Size of tree: " << treeSize << endl;
-*/
+
+//  treeSize = 0;
+//  kdDebug(24000) << "Basenode : " << m_node << endl;
+//  ParserCommon::coutTree(m_node, 2);
+//  kdDebug(24000) << "Size of tree: " << treeSize << endl;
+
 //FIXME: What is the use of two pointer to the same Node???
  baseNode = m_node;
+     kdDebug(24000) << "NN after parse = " << NN << "baseNode : " << baseNode << endl;
  m_saParser->init(m_node, w);
 
  //We need to reload Kafka to refresh the DOM::Node->Node links.
@@ -556,41 +571,7 @@ Node *Parser::parse(Document *w, bool force)
  return m_node;
 }
 
-/** Print the doc structure tree to the standard output.
-    Only for debugging purposes. */
 
-void Parser::coutTree(Node *node, int indent)
-{
-    QString output;
-    int bLine, bCol, eLine, eCol;
-    if (!node)
-        kdDebug(24000)<< "undoRedo::coutTree() - bad node!" << endl;
-    while (node)
-    {
-            output = "";
-            output.fill('.', indent);
-            node->tag->beginPos(bLine, bCol);
-            node->tag->endPos(eLine, eCol);
-            if (node->tag->type != Tag::Text)
-                    output += node->tag->name.replace('\n'," ");
-            else
-                    output+= node->tag->tagStr().replace('\n'," ");
-            kdDebug(24000) << output <<" (" << node->tag->type << ") at pos " <<
-                    bLine << ":" << bCol << " - " << eLine << ":" << eCol << " This: "<< node << " Parent: " << node->parent << " Prev: " << node->prev << " Next: " << node->next << " Child: " << node->child << endl;
- /*           for(j = 0; j < node->tag->attrCount(); j++)
-            {
-                    kdDebug(24000)<< " attr" << j << " " <<
-                            node->tag->getAttribute(j).nameLine << ":" <<
-                            node->tag->getAttribute(j).nameCol << " - " <<
-                            node->tag->getAttribute(j).valueLine << ":" <<
-                            node->tag->getAttribute(j).valueCol << endl;
-            }
-*/
-            if (node->child)
-                    coutTree(node->child, indent + 4);
-            node = node->next;
-    }
-}
 
 
 /** No descriptions */
@@ -691,6 +672,7 @@ Node *Parser::nodeAt(int line, int col, bool findDeepest, bool exact)
 }
 void Parser::logReparse(NodeModifsSet *modifs, Document *w)
 {
+
   NodeModif *modif;
   if (baseNode)
   {
@@ -830,6 +812,7 @@ void Parser::deleteNodes(Node *firstNode, Node *lastNode, NodeModifsSet *modifs)
   Node *node = firstNode;
   bool closesPrevious = false;
   NodeModif *modif;
+
   //delete all the nodes between the firstNode and lastNode
   while (node && node != lastNode )
   {
@@ -844,6 +827,10 @@ void Parser::deleteNodes(Node *firstNode, Node *lastNode, NodeModifsSet *modifs)
     {
       nextNode->prev = prev;
     }
+    if (nextNode && nextNode->parent == node)
+    {
+      nextNode->parent = parent;
+    }
     if (next)
       next->prev = prev;
     if (prev && prev->next == node)
@@ -863,8 +850,11 @@ void Parser::deleteNodes(Node *firstNode, Node *lastNode, NodeModifsSet *modifs)
     node->parent = 0L;
     node->next = 0L;
     node->prev = 0L;
+    
+    //delete node;
     node->detachNode();
     modif->setNode(node);
+
     node = 0L;
     i = 0;
     j = 0;
@@ -970,15 +960,17 @@ void Parser::deleteNodes(Node *firstNode, Node *lastNode, NodeModifsSet *modifs)
 
       }
     }
+
     modif->setChildrenMovedUp(i);
     modif->setNeighboursMovedDown(j);
     modifs->addNodeModif(modif);
+
     node = nextNode;
 
  //   kdDebug(24000)<< "Node removed!" << endl;
-//    coutTree(m_node, 2);
+//    ParserCommon::coutTree(m_node, 2);
   }
-//  coutTree(m_node, 2);
+//  ParserCommon::coutTree(m_node, 2);
 }
 
 Node *Parser::rebuild(Document *w)
@@ -995,13 +987,12 @@ Node *Parser::rebuild(Document *w)
  NodeModifsSet *modifs = new NodeModifsSet();
  NodeModif *modif;
 
-
 // kdDebug(24000)<< "Node *Parser::rebuild()" << endl;
  modifs->setIsModifiedAfter(w->isModified());
 
  //**kdDebug(24000)<< "************* Begin User Modification *****************" << endl;
   //debug!
-  //coutTree(m_node, 2);//*/
+  //ParserCommon::coutTree(m_node, 2);//*/
  if (w != write || !m_node) //the document has changed or the top node does not exists => parse completely
  {
   logReparse(modifs, w);
@@ -1023,15 +1014,17 @@ Node *Parser::rebuild(Document *w)
         (area.eLine < area.bLine || (area.eLine == area.bLine && area.eCol <= area.bCol)) //something strange has happened, like moving text with D&D inside the editor
       )
    {
-     logReparse(modifs, w);
-     m_saParser->setParsingEnabled(saParserEnabled);
-     return parse(w);
+      logReparse(modifs, w);
+       m_saParser->setParsingEnabled(saParserEnabled);
+       Node *n = parse(w, true);
+       return n;
    }
 
    kdDebug(24000) << QString("Invalid area: %1,%2,%3,%4").arg(area.bLine).arg(area.bCol).arg(area.eLine).arg(area.eCol) << "\n";
 
+//    kdDebug(24000) << "lastNode1: " << lastNode << " " << lastNode->tag << endl;
    deleteNodes(firstNode->nextSibling(), lastNode, modifs);
-
+//    kdDebug(24000) << "lastNode2: " << lastNode << " " << lastNode->tag << endl;
 
    firstNode->child = 0L;
    Node *lastInserted = 0L;
@@ -1070,10 +1063,11 @@ Node *Parser::rebuild(Document *w)
      m_saParser->setParsingEnabled(saParserEnabled);
      return parse(w);
    }
-
+//    kdDebug(24000) << "lastNode3: " << lastNode << " " << lastNode->tag << endl;
    bool goUp;
    if (lastNode && lastInserted)
    {
+//      kdDebug(24000) << "lastNode4: " << lastNode << " " << lastNode->tag << endl;
       //merge the nodes if they are both of type Text or Empty
       if ( (lastInserted->tag->type == Tag::Empty || lastInserted->tag->type == Tag::Text) &&
             (lastNode->tag->type == Tag::Empty || lastNode->tag->type == Tag::Text))
@@ -1105,6 +1099,7 @@ Node *Parser::rebuild(Document *w)
         modif = new NodeModif();
         modif->setType(NodeModif::NodeRemoved);
         modif->setLocation(kafkaCommon::getLocation(lastNode));
+
         if(lastInserted->prev)
           lastInserted->prev->next = 0L;
         if(lastInserted->parent && lastInserted->parent->child == lastInserted)
@@ -1113,6 +1108,8 @@ Node *Parser::rebuild(Document *w)
         lastInserted->next = 0L;
         lastInserted->parent = 0L;
         lastInserted->child = 0L;
+//        delete lastInserted;
+
         lastInserted->detachNode();
         modif->setNode(lastInserted);
         modifs->addNodeModif(modif);
@@ -1129,9 +1126,11 @@ Node *Parser::rebuild(Document *w)
 
     node = lastInserted;
 
+//     kdDebug(24000) << "lastNode5: " << lastNode << " " << lastNode->tag << endl;
     QTag *qTag = 0L;
     while (node && lastNode)
     {
+//       kdDebug(24000) << "lastNode6: " << lastNode << " " << lastNode->tag << endl;
       qTag = 0L;
       goUp = ( node->parent &&
                ( (lastNode->tag->type == Tag::XmlTagEnd && QuantaCommon::closesTag(node->parent->tag, lastNode->tag) ) ||
@@ -1156,77 +1155,82 @@ Node *Parser::rebuild(Document *w)
             (!m_dtd->caseSensitive && node->tag->name.lower() == node->parent->tag->name.lower())) )
           goUp = false; //it can happen that the tag closes the previous and not the parent
 
-    if (goUp) //lastnode closes the node->parent
-    {
-        //handle cases like <ul><li></ul>
-        if (lastNode->tag->type == Tag::XmlTagEnd &&
-            !QuantaCommon::closesTag(node->parent->tag, lastNode->tag))
-        {
-          while ( node->parent->parent &&
-                  QuantaCommon::closesTag(node->parent->parent->tag, lastNode->tag)
-                )
+      if (goUp) //lastnode closes the node->parent
+      {
+          //handle cases like <ul><li></ul>
+          if (lastNode->tag->type == Tag::XmlTagEnd &&
+              !QuantaCommon::closesTag(node->parent->tag, lastNode->tag))
           {
-            node = node->parent;
-          }
-        } else
-        if (qTag && lastNode->tag->type != Tag::XmlTagEnd)
-        {
-          //handle the case when a tag is a stopping tag for parent, and grandparent and so on. I'm not sure it's needed here, but anyway...
-          Node *n = node->parent;
-          QString searchFor = (m_dtd->caseSensitive) ? lastNode->tag->name : lastNode->tag->name.upper();
-          while (qTag && n)
-          {
-            qTag = QuantaCommon::tagFromDTD(m_dtd, n->tag->name);
-            if ( qTag )
+            while ( node->parent->parent &&
+                    QuantaCommon::closesTag(node->parent->parent->tag, lastNode->tag)
+                  )
             {
-              if ( qTag->stoppingTags.contains(searchFor) )
+              node = node->parent;
+            }
+          } else
+          if (qTag && lastNode->tag->type != Tag::XmlTagEnd)
+          {
+            //handle the case when a tag is a stopping tag for parent, and grandparent and so on. I'm not sure it's needed here, but anyway...
+            Node *n = node->parent;
+            QString searchFor = (m_dtd->caseSensitive) ? lastNode->tag->name : lastNode->tag->name.upper();
+            while (qTag && n)
+            {
+              qTag = QuantaCommon::tagFromDTD(m_dtd, n->tag->name);
+              if ( qTag )
               {
-                n->tag->closingMissing = true; //parent is single...
-                if (n->parent)
-                  node = n;
-                n = n->parent;
-              } else
-              {
-                break;
+                if ( qTag->stoppingTags.contains(searchFor) )
+                {
+                  n->tag->closingMissing = true; //parent is single...
+                  if (n->parent)
+                    node = n;
+                  n = n->parent;
+                } else
+                {
+                  break;
+                }
               }
             }
           }
-        }
-      if (lastNode->prev && lastNode->prev->next == lastNode)
-        lastNode->prev->next = 0L;
-      if (lastNode->parent && lastNode->parent->child == lastNode)
-          lastNode->parent->child = 0L;
-      node->parent->next = lastNode;
-      lastNode->prev = node->parent;
-      if (node->parent)
-        lastNode->parent = node->parent->parent;
-      else
-        lastNode->parent = 0L;
-      node->next = 0L;
-      lastNode->closesPrevious = true;
-    } else
-    {
-      if (lastNode->prev && lastNode->prev->next == lastNode)
+        if (lastNode->prev && lastNode->prev->next == lastNode)
           lastNode->prev->next = 0L;
-      node->next = lastNode;
-      lastNode->prev = node;
-      lastNode->parent = node->parent;
+        if (lastNode->parent && lastNode->parent->child == lastNode)
+            lastNode->parent->child = 0L;
+        if (node->parent)
+          node->parent->next = lastNode;
+        lastNode->prev = node->parent;
+        if (node->parent)
+          lastNode->parent = node->parent->parent;
+        else
+          lastNode->parent = 0L;
+        node->next = 0L;
+        lastNode->closesPrevious = true;
+      } else
+      {
+        if (lastNode->prev && lastNode->prev->next == lastNode)
+            lastNode->prev->next = 0L;
+        node->next = lastNode;
+        lastNode->prev = node;
+        lastNode->parent = node->parent;
+//         kdDebug(24000) << "lastNode7: " << lastNode << " " << lastNode->tag << endl;
+      }
+      node = lastNode;
+      lastNode = lastNode->nextNotChild();
+      if (lastNode)
+        QString s = lastNode->tag->tagStr();
     }
-    node = lastNode;
-    lastNode = lastNode->nextNotChild();
-   }
  }
 /*   kdDebug(24000)<< "END"<< endl;
-   coutTree(baseNode,  2);
+   ParserCommon::coutTree(baseNode,  2);
    kdDebug(24000)<< "************* End User Modification *****************" << endl;*/
+
    w->docUndoRedo->addNewModifsSet(modifs, undoRedo::SourceModif);
  }
-  kdDebug(24000) << "Rebuild: " << t.elapsed() << " ms \n";
-/*
- treeSize = 0;
- coutTree(m_node, 2);
- kdDebug(24000) << "Size of tree: " << treeSize << endl;
-// cout << "\n"; */
+  kdDebug(24000) << "Rebuild: " << t.elapsed() << " ms; baseNode=" << baseNode << "\n";
+ 
+//   ParserCommon::verifyTree(m_node);
+/* treeSize = 0;
+ ParserCommon::coutTree(m_node, 2);
+ kdDebug(24000) << "Size of tree: " << treeSize << endl;*/
 
  m_saParser->init(m_node, w);
  if (saParserEnabled)
@@ -1239,7 +1243,7 @@ Node *Parser::rebuild(Document *w)
 void Parser::clearGroups()
 {
 #ifdef DEBUG_PARSER
-  kdDebug(24000) << "clearGroups " << endl;
+//   kdDebug(24000) << "clearGroups " << endl;
 #endif
   GroupElementMapList::Iterator it;
   GroupElementList::Iterator elementIt;
@@ -1274,7 +1278,7 @@ void Parser::clearGroups()
     }
   }
 #ifdef DEBUG_PARSER
-      kdDebug(24000) << count << " GroupElement deleted (clearGroups)." << endl;
+//       kdDebug(24000) << count << " GroupElement deleted (clearGroups)." << endl;
 #endif
   globalGroupMap.clear();
   clearIncludedGroupElements();
@@ -1290,7 +1294,7 @@ void Parser::clearGroups()
 void Parser::cleanGroups()
 {
 #ifdef DEBUG_PARSER
-  kdDebug(24000) << "cleanGroups " << endl;
+//   kdDebug(24000) << "cleanGroups " << endl;
 #endif
   GroupElementMapList::Iterator it;
   GroupElementList::Iterator elementIt;
@@ -1325,7 +1329,7 @@ void Parser::cleanGroups()
     }
   }
 #ifdef DEBUG_PARSER
-      kdDebug(24000) << count << "GroupElement deleted (cleanGroups)." << endl;
+//       kdDebug(24000) << count << " GroupElement deleted (cleanGroups)." << endl;
 #endif
   if (m_parseIncludedFiles)
   {
@@ -1731,7 +1735,7 @@ void Parser::slotParseInDetail()
 
 void Parser::synchParseInDetail()
 {
-  m_saParser->parseInDetail(false);
+  m_saParser->parseInDetail(true);
 }
 
 void Parser::setSAParserEnabled(bool enabled)
